@@ -335,15 +335,18 @@ void MainWindow::onChangeUpnp() {
 }
 
 void MainWindow::showInstallButton(Gtk::Button *button, bool available) {
-  if (available || !packageKitAvailable)
-    button->hide();
-  else
-    button->show();
+    if (available || !packageKitAvailable)
+        button->hide();
+    else
+        button->show();
 }
 
 void MainWindow::installFiles(const char *a, const char *b = NULL) {
     Glib::RefPtr<Gio::DBus::Proxy> proxy;
-    const std::vector<Glib::ustring> packages = {a, b};
+    std::vector<Glib::ustring> packages = {a};
+
+    if (b)
+        packages.push_back(Glib::ustring(b));
 
     proxy = Gio::DBus::Proxy::create_for_bus_sync(Gio::DBus::BusType::BUS_TYPE_SESSION,
                                                       "org.freedesktop.PackageKit",
@@ -367,16 +370,16 @@ void MainWindow::installFiles(const char *a, const char *b = NULL) {
 }
 
 void MainWindow::installModules(const char *a, const char *b = NULL) {
-  gchar *ma, *mb = NULL;
+    gchar *ma, *mb = NULL;
 
-  ma = modulePath (a);
-  if (b != NULL)
-    mb = modulePath (b);
+    ma = modulePath (a);
+    if (b != NULL)
+        mb = modulePath (b);
 
-  installFiles (ma, mb);
+    installFiles (ma, mb);
 
-  g_free (ma);
-  g_free (mb);
+    g_free (ma);
+    g_free (mb);
 }
 
 void MainWindow::onZeroconfDiscoverInstallButtonClicked() {
@@ -670,38 +673,76 @@ void MainWindow::readFromGSettings() {
 }
 
 gchar * MainWindow::modulePath(const gchar *name) {
-  gchar *path, **versions;
+    /* PulseAudio 16.0 dropped the version number from the module directory.
+     * The old module directory looked like "${libdir}/pulse-X.Y/modules", the
+     * new directory looks like "${libdir}/pulseaudio/modules". To deal with
+     * the change, we have three cases:
+     *
+     * 1) If MODDIR ends with "/pulseaudio/modules", we assume it's can be used
+     *    as is.
+     *
+     * 2) Otherwise paprefs might be built against an older PulseAudio version,
+     *    but the running PulseAudio version might be newer, so we check if
+     *    "${libdir}/pulseaudio/modules" exists, in which case we use it. We
+     *    don't actually use libdir in the check, though, we use MODDIR with
+     *    the last two components removed (the removed part is assumed to be
+     *    "/pulse-X.Y/modules"). In the usual case that will be the same as
+     *    libdir, but PulseAudio might be built with a custom module directory,
+     *    so it's better to use MODDIR.
+     *
+     * 3) Otherwise we assume that MODDIR ends with "/pulse-X.Y/modules" and we
+     *    use that, except we replace the version number with the version
+     *    reported by pa_get_library_version().
+     *
+     * FIXME: It would be better to connect to the PulseAudio server and ask
+     * its version, but that would require adding a "connecting to PulseAudio"
+     * phase to startup (requiring UI changes), so implementing it requires
+     * more work. */
 
-  versions = g_strsplit(pa_get_library_version(), ".", 3);
-  if (versions[0] && versions[1]) {
-      gchar *pulsedir, *search;
+    gchar *libdir = NULL, *search, *moddir = NULL, *path, **versions;
 
-      /* Remove the "/pulse-x.y/modules" suffix so we can dynamically inject
-       * it again with runtime library version numbers */
-      pulsedir = g_strdup_printf ("%s", MODDIR);
-      if ((search = g_strrstr (pulsedir, G_DIR_SEPARATOR_S))) {
-          *search = '\0';
-          if ((search = g_strrstr (pulsedir, G_DIR_SEPARATOR_S)))
-              *search = '\0';
-      }
-      path = g_strdup_printf ("%s" G_DIR_SEPARATOR_S "pulse-%s.%s" G_DIR_SEPARATOR_S "modules" G_DIR_SEPARATOR_S "%s", pulsedir, versions[0], versions[1], name);
-      g_free (pulsedir);
-  } else
-      path = g_build_filename (MODDIR, name, NULL);
-  g_strfreev(versions);
+    /* Case 1: */
+    if (g_str_has_suffix(MODDIR, G_DIR_SEPARATOR_S "pulseaudio" G_DIR_SEPARATOR_S "modules"))
+        return g_build_filename(MODDIR, name, NULL);
 
-  return path;
+    /* Case 2: */
+    libdir = g_strdup(MODDIR);
+    if ((search = g_strrstr(libdir, G_DIR_SEPARATOR_S))) {
+        *search = '\0';
+        if ((search = g_strrstr(libdir, G_DIR_SEPARATOR_S)))
+            *search = '\0';
+    }
+
+    moddir = g_build_filename(libdir, "pulseaudio", "modules", NULL);
+    if (g_file_test(moddir, G_FILE_TEST_IS_DIR)) {
+        path = g_build_filename(moddir, name, NULL);
+        goto finish;
+    }
+
+    /* Case 3: */
+    versions = g_strsplit(pa_get_library_version(), ".", 3);
+    if (versions[0] && versions[1])
+        path = g_strdup_printf("%s" G_DIR_SEPARATOR_S "pulse-%s.%s" G_DIR_SEPARATOR_S "modules" G_DIR_SEPARATOR_S "%s", libdir, versions[0], versions[1], name);
+    else
+        path = g_build_filename (MODDIR, name, NULL);
+    g_strfreev(versions);
+
+finish:
+    g_free(moddir);
+    g_free(libdir);
+
+    return path;
 }
 
 bool MainWindow::moduleExists(const gchar *name) {
-  gchar *path = modulePath (name);
-  bool ret;
+    gchar *path = modulePath (name);
+    bool ret;
 
-  ret = g_file_test (path, G_FILE_TEST_EXISTS);
+    ret = g_file_test (path, G_FILE_TEST_EXISTS);
 
-  g_free (path);
+    g_free (path);
 
-  return ret;
+    return ret;
 }
 
 void MainWindow::checkForModules() {
